-------------------------------------------------------------------------------
-- Title			:	Triangular Wave Generator in FPGA.
-- Project		:	Sample code
-- File			:	triangular_wave.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	It generates binary o/p for triangular wave. 
--					User can connect its o/p to DAC to check
--					on CRO.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity tri is
    Port ( rst : in std_logic;
           clk : in std_logic;
           dac : out std_logic_vector(7 downto 0));
end tri;

architecture Behavioral of tri is
signal cnt : integer range 0 to 255;
signal dac_2s : std_logic_vector(7 downto 0);
signal u_d,tc : std_logic ;
begin


-- Up/Down Binary Counter
process (rst,clk,cnt)
begin 
	if (rst = '0') then
		cnt <= 1 ;
	elsif (clk'event and clk = '1') then
  		if (u_d = '0') then
			cnt <= cnt +1;
		else
			cnt <= cnt - 1;
		end if;
	end if;
	end process;
	
-- Terminal Count
tc	<= '1' when cnt=1 or cnt=255 else
	   '0';

-- Direction Signal Generation for Counter
process(rst,tc,u_d)
begin
if(rst ='0')then
	u_d <= '0';
elsif(tc'event and tc ='1')then
	u_d <= not u_d;
end if;
end process;

-- Binary O/P
dac_2s <= conv_std_logic_vector(cnt,8);

-- Signed Binary O/P
dac <= (not dac_2s) + 1;

end Behavioral;